/* ELPDateTime.rex */

    .local~check.dtypes = 'B D E M N O S U W'
    .local~check.months = 'January February March April May June' -
                          'July August September October November December'
    .local~check.mon    = 'Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec'
    .local~check.count  = '31 29 31 30 31 30 31 31 30 31 30 31'
    .local~check.days   = 'Monday Tuesday Wednesday Thursday Friday Saturday'
    .local~check.ttypes = 'C H L M N S'
    .local~check.ap     = 'am pm'

::Routine RxIsTime public
    use arg ctime, type
    type= type~translate
    estring = 'Invalid Time ->'
    select
        when .check.ttypes~wordpos(type) < 1 then
            estring = 'Time Validation of Type' type 'Is Not Supported'
        when TimeType(ctime,type) then
            estring = 1
        when type = 'C' then do
            parse var ctime hh':'mm
            ap = mm~right(2)
            mm = mm~left(2)
            select
                when ctime~pos(':') < 1 then
                    estring = estring 'Type C Must Contain A Colon (:) Hour Minute Seperator'
                when hh = '' | mm = '' then
                    estring = estring 'Type C Must Be In Format hH:MMam/pm'
                when .check.ap~wordpos(ap) < 1 then
                    estring = estring 'Type C Must Be Have A Suffix Of Either "am" OR "pm"'
                when hh < 1 | hh > 23 then
                    estring = estring 'Hours Must Be Between 1 & 23'
                when mm < 0 | mm > 59 then
                    estring = estring 'Minutes Must Be Between 0 & 59'
                otherwise
                    estring = estring 'Hell If I Know Why'
            end
        end
        when type = 'H' then
            estring = estring 'Hours Must Be Between 0 & 23'
        when type = 'L' then do
            parse var ctime hh':'mm':'ss'.'dd
            select
                when ctime~length \= 15 | ctime~countstr(':') < 2 | time~countstr('.') < 1 then
                    estring = estring 'Type L Must Be In The Format HH:MM:SS.DDDDDD'
                when hh~length < 2 | hh < 0 | hh > 23 then
                    estring = estring 'HH Must Be 2 Digits Between 00 & 23'
                when mm~length < 2 | mm < 0 | mm > 59 then
                    estring = estring 'MM Must Be 2 Digits Between 00 & 59'
                when ss~length < 2 | ss < 0 | ss > 59 then
                    estring = estring 'SS Must Be 2 Digits Between 00 & 59'
                when dd~length < 6 | dd < 0 | dd > 999999 then
                    estring = estring 'DDDDDD Must Be 6 Digits Between 000000 & 999999'
            end
        end
        when type = 'M' then
            estring = estring 'Minutes Must Be Between 0 & 1439'
        when type = 'N' then do
            parse var ctime hh':'mm':'ss
            select
                when ctime~length \= 8 | ctime~countstr(':') < 2 then
                    estring = estring 'Type N Must Be In The Format HH:MM:SS'
                when hh~length < 2 | hh < 0 | hh > 23 then
                    estring = estring 'HH Must Be 2 Digits Between 00 & 23'
                when mm~length < 2 | mm < 0 | mm > 59 then
                    estring = estring 'MM Must Be 2 Digits Between 00 & 59'
                when ss~length < 2 | ss < 0 | ss > 59 then
                    estring = estring 'SS Must Be 2 Digits Between 00 & 59'
            end
        end
        when type = 'S' then
            estring = estring 'Minutes Must Be Between 0 & 86399'
        otherwise
            nop  -- Already validated the type, so can't (shouldn't) get here
    end
return estring

::Routine RxIsDate public
    use arg cdate, type
    type = type~translate
    estring = 'Invalid Date ->'
    select
        when .check.dtypes~wordpos(type) < 1 then
            estring = 'Date Validation of Type' type 'Is Not Supported'
        when DateType(cdate,type) then
            estring = 1
        when type = 'B' then
            estring = 'Invalid Date --> A Base Date Must Be Between 0 & 999999999.'
        when type = 'D' then do
            max = date('d',date('s')~left(4)||1231,'s')
            select
                when \cdate~datatype('w') then
                    estring = estring 'Date Must Be A Positive Integer Greater Than 0'
                when cdate < 1 | cdate > max then
                    estring = estring 'The Day Must Be Between 1 &' max
                otherwise
                    estring = estring 'Hell If I Know Why'
            end
        end
        when type = 'E' then do
            parse var cdate dd'/'mm'/'yy
            select
                when cdate~length \= 8 then
                    estring = estring 'Must Be 8 Digits In The Format DD/MM/YY'
                when cdate~substr(3,1) \= '/' | cdate~substr(6,1) \= '/' then
                    estring = estring 'DD MM YY Must Be Delimeted With A Forward Slash (/)'
                when \dd~datatype('w') | dd~sign \= 1 then
                    estring = estring 'DD Must Be A Positive Integer Greater Than 0'
                when \mm~datatype('w') | mm~sign \= 1 then
                    estring = estring 'MM Must Be A Positive Integer Greater Than 0'
                when \yy~datatype('w') | yy~sign \= 1 then
                    estring = estring 'YY Must Be A Positive Integer'
                when mm < 1 | mm > 12 then
                    estring = estring 'MM Must Be Between 1 & 12'
                when mm = 2 then do
                    if date('d','31/12/'yy,'e') = 366 then max = 29
                    else max = 28
                    if dd < 1 | dd > max then
                        estring = estring 'Valid DD For' mm 'Must Be Between 1 &' max
                end
                when mm \= 2 then do
                    max = .check.count~word(mm)
                    estring = estring 'Valid DD For' mm 'Must Be Between 1 &' max
                end
                otherwise
                    estring = estring 'Hell If I Know Why'
            end
        end
        when type = 'M' then do
            if .check.months~wordpos(cdate) < 1 then
                estring = estring cdate 'Is Not A Valid Month'
        end
        when type = 'N' then do
            dd = cdate~word(1)
            mm = cdate~word(2)
            yy = cdate~word(3)
            select
                when cdate~words \= 3  then
                    estring = estring 'Your Date Must Be In The Format D Mon YYYY'
                when .check.mon~wordpos(mm) < 1 then
                    estring = estring cdate~word(2) 'Is Not A Valid Month Abbreviation'
                when \dd~datatype('w') | dd~sign \= 1 then
                    estring = estring 'Your Day Must Be A Positive Integer Greater Than 0'
                when \yy~datatype('w') | yy~sign \= 1 then
                    estring = estring 'Your Year Must Be A Positive Integer'
                when .check.mon~wordpos(mm) = 2 then do
                    if date('d','31/12/'yy,'e') = 366 then max = 29
                    else max = 28
                    if dd < 1 | dd > max then
                        estring = estring 'Valid DD For' mm 'Must Be Between 1 &' max
                end
                when .check.mon~wordpos(mm) \= 2 then do
                    max = .check.count~word(.check.mon~wordpos(mm))
                    estring = estring 'Valid DD For' mm 'Must Be Between 1 &' max
                end
                when \yy~datatype('w') | yy~sign \= 1 | yy < 1 then
                    estring = estring 'Year Must Be A Whole Number Between 0000 & 9999'
                otherwise
                    estring = estring 'Hell If I Know Why'
            end
        end
        when type = 'O' then do
            parse var cdate yy'/'mm'/'dd
            select
                when cdate~length \= 8 then
                    estring = estring 'Must Be 8 Digits In The Format YY/MM/DD'
                when cdate~substr(3,1) \= '/' | cdate~substr(6,1) \= '/' then
                    estring = estring 'YY MM DD Must Be Delimeted With A Forward Slash (/)'
                when \dd~datatype('w') | dd~sign \= 1 then
                    estring = estring 'DD Must Be A Positive Integer Greater Than 0'
                when \mm~datatype('w') | mm~sign \= 1 then
                    estring = estring 'MM Must All A Positive Integer Greater Than 0'
                when \yy~datatype('w') | yy~sign \= 1 then
                    estring = estring 'YY Must Be A Positive Integer'
                when mm < 1 | mm > 12 then
                    estring = estring 'MM Must Be Between 1 & 12'
                when mm = 2 then do
                    if date('d','31/12/'yy,'e') = 366 then max = 29
                    else max = 28
                    if dd < 1 | dd > max then
                        estring = estring 'Valid DD For' mm 'Must Be Between 1 &' max
                end
                when mm \= 2 then do
                    max = .check.count~word(mm)
                    estring = estring 'Valid DD For' mm 'Must Be Between 1 &' max
                end
                otherwise
                    estring = estring 'Hell If I Know Why'
            end
        end
        when type = 'S' then do
            yy = cdate~left(4)
            mm = cdate~substr(5,2)
            dd = cdate~right(2)
            select
                when cdate~length \= 8 then
                    estring = estring 'Must Be 8 Digits In The Format YYYYMMDD'
                when \cdate~datatype('w') | cdate~sign \= 1 then
                    estring = estring 'Your Date Must Be A Whole Positive Number'
                when mm < 1 | mm > 12 then
                    estring = estring mm 'Is Not A Valid Month'
                when yy < 1 | yy > 9999 then
                    estring = estring yy 'Is Not A Valid Year'
                when dd < 1 then
                    estring = estring yy 'Is Not A Valid Day'
                when mm = 2 then do
                    yyyy = cdate~left(4)
                    if date('d',yyyy'1231','s') = 366 then max = 29
                    else max = 28
                    if dd < 1 | dd > max then
                        estring = estring 'Valid Day For' mm 'Must Be Between 1 &' max
                end
                when mm \= 2 then do
                    max = .check.count~wordpos(mm)
                    estring = estring 'Valid Day For' mm 'Must Be Between 1 &' max
                end
                otherwise
                    estring = estring 'Hell If I Know Why'
            end

        end
        when type = 'U' then do
            parse var cdate mm'/'dd'/'yy
            select
                when cdate~length \= 8 then
                    estring = estring 'Must Be 8 Digits In The Format MM/DD/YY'
                when cdate~substr(3,1) \= '/' | cdate~substr(6,1) \= '/' then
                    estring = estring 'MM DD YY Must Be Delimeted With A Forward Slash (/)'
                when \dd~datatype('w') | dd~sign \= 1 then
                    estring = estring 'DD Must All Be A Positive Integer Greater Than 0'
                when \mm~datatype('w') | mm~sign \= 1 then
                    estring = estring 'MM Must All Be A Positive Integer Greater Than 0'
                when \yy~datatype('w') | yy~sign \= 1 then
                    estring = estring 'YY Must Be A Positive Integer'
                when mm < 1 | mm > 12 then
                    estring = estring 'MM Must Be Between 1 & 12'
                when mm = 2 then do
                    if date('d','31/12/'yy,'e') = 366 then max = 29
                    else max = 28
                    if dd < 1 | dd > max then
                        estring = estring 'Valid DD For' mm 'Must Be Between 1 &' max
                end
                when mm \= 2 then do
                    max = .check.count~word(mm)
                    estring = estring 'Valid DD For' mm 'Must Be Between 1 &' max
                end
                otherwise
                    estring = estring 'Hell If I Know Why'
            end
        end
        when type = 'W' then
            if .check.days~wordpos(cdate) < 1 then
                estring = estring cdate 'Is Not A Valid Day Of The Week'
        otherwise
            nop  -- Already validated the type, so can't (shouldn't) get here
    end
return estring

::Routine DateType public
    use arg cdate, type
    signal on syntax name DateSyntax
    select
        when type~translate = 'L' then signal DateSyntax  --Special Case For Tyepe "L"
        when type~translate = 'M' then                    --Special Case For Type 'M'
            if .check.months~wordpos(cdate) < 1 then signal DateSyntax
        when type~translate = 'W' then                    --Special Case For Type 'W'
            if .check.days~wordpos(cdate) < 1 then signal DateSyntax
        otherwise
            call date type, cdate, type
    end
return .true

DateSyntax:
    return .false

::Routine TimeType public
    use arg ctime, type
    signal on syntax name TimeSyntax
    call time type, ctime, type
    return .true

TimeSyntax:
    return .false


