/* RxXL_Automation_001.rex */
    parse source src
    d_title = substr(src,src~lastpos('\')+1,src~length-src~lastpos('\'))

/*
The purpose of this application is to demonstrate the use of ooRexx ActiveX/OLE to automate the
creation of an Excel Workbook.  The Excel Workbook created in this application will be used in
the next application in this series that demonstartes the extraction of data from an existing
Excel workbook.

*/
    slowdown = .true

--Clear the screen
    call SysCls

-- Create the Excel object
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Create the Excel object')
            smsg~Execute('ShowTop')
        end
    xlobj = .OleObject~New('Excel.Application')

/*Turn the visible attribute on so that we can see what is happening
  In a production application, this can be set to false */
    xlobj~Visible = .true

--Create a new workbook with 1 worksheet
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Create a workbook with 1 worksheet')
            smsg~Execute('ShowTop')
        end
    xlobj~Application~SheetsInNewWorkBook = 1
    xlobj~WorkBooks~Add

/*For the purpose of demonstration we will load random numbers into our workbook from
  columns A-H on rows 3-25.
  Row 1 will hold our "title line"
  Row 2 will hold our "headers"
  Each random number will be divided by 100 and formatted to 2 decimal places
  */
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Add some random numbers to a range of cells')
            smsg~Execute('ShowTop')
        end
    do row = 3 to 25
      do column = 1 to 8
          select
              when column = 1 then
                  xlobj~Cells(row,column)~Value = random(1,3)
              when column = 2 then
                  xlobj~Cells(row,column)~Value = random(8,10)
              when column = 3 then
                  xlobj~Cells(row,column)~Value = random(1,5)
              otherwise
                  xlobj~Cells(row,column)~Value = (random(0,100000) / 100)~format(,2)

          end
      end
  end

/*We know our data starts in Column A - Row 3
  Lets assume that we have no idea where our data ends
  To determine where our data ends we will need to use an Excel constant */
    xlLastCell = xlobj~GetConstant('xlLastCell')

--Now that we have the constant value, lets use it to determine our last used cell
    lastcell = xlobj~ActiveCell~SpecialCells(xlLastCell)~Address

/*The value in lastcell will be something like $c$r (where c indicates the column and
  r indicates the row).
  So we will need to use the parse statement to breakout our max column and max row */
    parse var lastcell '$'max_column'$'max_row

/*Now that we know where our data starts and where it ends, lets select this range
  and format all cells to 2 decimal places and draw boxes around the cells */

--Select the range
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Select the range we just loaded')
            smsg~Execute('ShowTop')
        end
    xlobj~Range('A3:'max_column||max_row)~Select

--Set numeric format
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Set the numeric format')
            smsg~Execute('ShowTop')
        end
    mySelection = xlobj~Selection
    mySelection~NumberFormat = "#0.00"

--Draw boxes around cells
--We'll need some more constants here, so lets retrieve them first
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Draw boxes around cells')
            smsg~Execute('ShowTop')
        end
    xlContinuous = xlobj~GetConstant('xlContinuous')
    xlThin = xlobj~GetConstant('xlThin')
    xlAutomatic = xlobj~GetConstant('xlAutomatic')

--Use the constants to draw boxes
    mySelection~Borders~LineStyle = xlContinuous
    mySelection~Borders~Weight = xlThin
    mySelection~Borders~ColorIndex = xlAutomatic

--Now lets go back and place some headers on Row 2
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Add some column headers')
            smsg~Execute('ShowTop')
        end
    do column = 1 to 8
        if column = 8 then
            xlobj~Cells(2,column)~Value = 'BigWideHeader'column
        else
            xlobj~Cells(2,column)~Value = 'Header'column
    end

--Define a new Range selection that is only row 2
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Draw boxes around the headers')
            smsg~Execute('ShowTop')
        end
    xlobj~Range('A2:'max_column||2)~Select
    mySelection = xlobj~Selection

/*Now that we have the new selection, lets draw boxes around it and center the
  contents of the cells*/
--Note, we already have our constants retrieved so we don't have to retrieve them again
    mySelection~Borders~LineStyle = xlContinuous
    mySelection~Borders~Weight = xlThin
    mySelection~Borders~ColorIndex = xlAutomatic

--Now lets center the cell data (we'll need another constant)
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Center the headers')
            smsg~Execute('ShowTop')
        end
    xlCenter = xlobj~GetConstant(xlCenter)
    mySelection~HorizontalAlignment = xlCenter

/*To keep from wasting space, lets autofit the columns
  We know that column A is the first column and the last column is now stored in max_column
  so lets select just the "columns" */
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Autofit the columns')
            smsg~Execute('ShowTop')
        end
    xlobj~Range('A:'max_column)~Select
    mySelection = xlobj~Selection
--Now autofit the columns
    mySelection~Columns~AutoFit

/*Now that we have our data and headers stored in the cells, lets go back to row 1 and
  add a title line */
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Add a title on row 1')
            smsg~Execute('ShowTop')
        end
    xlobj~Cells(1,'A')~Value = 'Our First Automated Excel WorkBook'

/*Now that we have a title, lets "merge" it across all the columns, change the font size,
  and the font color */

--Merge & center it first
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Merge and center the title')
            smsg~Execute('ShowTop')
        end
    xlobj~Range('A1:'max_column||1)~Select
    mySelection = xlobj~Selection
    mySelection~HorizontalAlignment = xlCenter
    mySelection~Merge

--Now lets set the font size
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Adjust the font name, size, and color of the title')
            smsg~Execute('ShowTop')
        end
    mySelection~Font~Name = 'Arial'
    mySelection~Font~Size = 14

--Set the font color to red
    mySelection~Font~ColorIndex = 3

/*Now that we have finished creating our workbook, lets give it a name and save it
  we'll save it in the default Excel folder (which will probably be My Documents'
  NOT the folder that this application is running in - you can provide any path you'd like */
    outfile = '.\RxXL_Automation_001.xls'

/*We may want to run this several times and Excel will prompt us about replacing an existing
  file.  Since this conflicts with "automation" we'll need to turn off the Excel warnings */
    xlobj~DisplayAlerts = .false

--Save the file
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Save the workbook')
            smsg~Execute('ShowTop')
        end
    xlobj~ActiveSheet~SaveAs(outfile)

--Close the Workbook
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Close the workbook')
            smsg~Execute('ShowTop')
        end
    xlobj~WorkBooks(1)~Close(SaveAll)

--Quit Excel
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Quit Excel')
            smsg~Execute('ShowTop')
        end
    xlobj~Quit

    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Congratulations')
            smsg~Execute('ShowTop')
        end
exit

::requires 'oodwin32.cls'
::class infoDlg subclass userdialog public
::method Init
    expose width height d_title d_text
    use arg d_title, d_text
    self~Init:super()
    width = 350
    height = 45
    rc=self~CreateCenter(width,height,d_title,,,'Arial',10)

::method DefineDialog
    expose width height d_title d_text
    forward class(super) continue
    self~AddButton( 1,width-60 ,height-20,50,15,'OK','Ok','DEFAULT')
    self~AddText(1,5,350,15,d_text)

::method Ok
    self~OK:super
return self~finished


