/* RxXL_Automation_002.rex */
    parse source src
    d_title = substr(src,src~lastpos('\')+1,src~length-src~lastpos('\'))

/*
The purpose of this application is to demonstrate the use of ooRexx ActiveX/OLE to automate the
reading of an Excel Workbook.  The Excel Workbook that will be read by this application should
have already been created by running RxXL_Automation_001.rex
*/
    slowdown = .true

--Clear the screen
    call SysCls

--We'll use a tab to line up our "say" data
    tab = '09'x

--Create a "word" list of columns - you'll see where we need this later
    column_list='A B C D E F G H I J K L M N O P Q R S T U V W X Y Z',
                'AA AB AC AD AE AF AG AH AI AJ AK AL AM AN AO AP AQ AR AS AT AU AV AW AX AY AZ',
                'BA BB BC BD BE BF BG BH BI BJ BK BL BM BN BO BP BQ BR BS BT BU BV BW BX BY BZ',
                'CA CB CC CD CE CF CG CH CI CJ CK CL CM CN CO CP CQ CR CS CT CU CV CW CX CY CZ',
                'DA DB DC DD DE DF DG DH DI DJ DK DL DM DN DO DP DQ DR DS DT DU DV DW DX DY DZ',
                'EA EB EC ED EE EF EG EH EI EJ EK EL EM EN EO EP EQ ER ES ET EU EV EW EX EY EZ',
                'FA FB FC FD FE FF FG FH FI FJ FK FL FM FN FO FP FQ FR FS FT FU FV FW FX FY FZ',
                'GA GB GC GD GE GF GG GH GI GJ GK GL GM GN GO GP GQ GR GS GT GU GV GW GX GY GZ',
                'HA HB HC HD HE HF HG HH HI HJ HK HL HM HN HO HP HQ HR HS HT HU HV HW HX HY HZ',
                'IA IB IC ID IE IF IG IH II IJ IK IL IM IN IO IP IQ IR IS IT IU IV'


-- Create the Excel object
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Create the Excel object')
            smsg~Execute('ShowTop')
        end
    xlobj = .OleObject~New('Excel.Application')

/*Turn the visible attribute on so that we can see what is happening
  In a production application, this can be set to false */
    xlobj~Visible = .true

--Open an existing workbook
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Open an existing workbook')
            smsg~Execute('ShowTop')
        end
    infile = '.\RxXL_Automation_001.xls'
    xlobj~WorkBooks~Open(infile)

/*At this point lets assume the only thing we know about the WorkBook is that "real" data begins
  on Row 3 Column A*/

--We need to determine our lastcell (we'll need the constant to do that)
    xlLastCell = xlobj~GetConstant('xlLastCell')
    lastcell = xlobj~ActiveCell~SpecialCells(xlLastCell)~Address
    parse var lastcell '$'max_column'$'max_row

--Use the column_list from above to determine the "number" of the last column
    max_column_number = column_list~wordpos(max_column)

/*At this point we should know that our data falls in the Range A3 & max_column_number||max_row
  For purposes of this demonstartion, we will simply say the cell # and the value in it.
  A real application would save this data in possibly a stem variable or a collection. */

    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Read the cell contents')
            smsg~Execute('ShowTop')
        end
    do column = 1 to max_column_number
        do row = 3 to max_row
            cell_value = xlobj~Cells(row,column)~Value
            column_letter = column_list~word(column)
            say 'Row' row tab 'Column' column_letter '=' cell_value
        end
    end

--Lets close the workbook - no need to save since all we did was read data
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Close the workbook - NO save')
            smsg~Execute('ShowTop')
        end
    xlobj~ActiveWorkbook~Close

--Quit Excel
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Quit Excel')
            smsg~Execute('ShowTop')
        end
    xlobj~Quit

    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Your cell contents are displayed in the command prompt window')
            smsg~Execute('ShowTop')
        end
exit

::requires 'oodwin32.cls'
::class infoDlg subclass userdialog public
::method Init
    expose width height d_title d_text
    use arg d_title, d_text
    self~Init:super()
    width = 350
    height = 45
    rc=self~CreateCenter(width,height,d_title,,,'Arial',10)

::method DefineDialog
    expose width height d_title d_text
    forward class(super) continue
    self~AddButton( 1,width-60 ,height-20,50,15,'OK','Ok','DEFAULT')
    self~AddText(1,5,350,15,d_text)

::method Ok
    self~OK:super
return self~finished




