/* RxXL_Automation_004.rex */
    parse source src
    d_title = substr(src,src~lastpos('\')+1,src~length-src~lastpos('\'))

/*
The purpose of this application is to demostrate the use of ooRexx ActiveX/OLE to automate the
sorting of an Excel Workbook.  The Excel Workbook that will be read by this application should
have already been created by running RxXL_Automation_001.rex
This will sort by column A ascending
Within that sort we'll sort column B descending
*/
    slowdown = .true

--Clear the screen
    call SysCls

-- Create the Excel object
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Create the Excel object')
            smsg~Execute('ShowTop')
        end
    xlobj = .OleObject~New('Excel.Application')

/*Turn the visible attribute on so that we can see what is happening
  In a production application, this can be set to false */
    xlobj~Visible = .true

--Open an existing workbook
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Open an existing workbook')
            smsg~Execute('ShowTop')
        end
    infile = '.\RxXL_Automation_001.xls'
    myWorkBook = xlobj~WorkBooks~Open(infile)

/*At this point lets assume the only thing we know about the WorkBook is that "real" data begins
  on Row 3 Column A*/

--We need to determine our lastcell (we'll need the constant to do that)
    xlLastCell = xlobj~GetConstant('xlLastCell')
    lastcell = xlobj~ActiveCell~SpecialCells(xlLastCell)~Address
    parse var lastcell '$'max_column'$'max_row

--Select the range to be sorted
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Select the range to sort')
            smsg~Execute('ShowTop')
        end
    xlobj~Range('A3:'max_column||max_row)~Select
    mySelection = xlobj~Selection
    mySheet = myWorkBook~ActiveSheet

--We need a few constants
    xlNormal = xlobj~GetConstant(xlNormal)
    xlAscending = xlobj~GetConstant('xlAscending')
    xlDescending = xlobj~GetConstant('xlDescending')

--Do the sort
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Perform the sort')
            smsg~Execute('ShowTop')
        end
    mySelection~Sort(mySheet~Range('A3'),xlAscending,mySheet~Range('B3'),,xlDescending)
--The key to making this work is the extra parameter--------------------^

--Be nice and place the cursor in cell A1
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Place cursor in A1')
            smsg~Execute('ShowTop')
        end
    xlobj~Range('A1')~Select

/*Now that we have finished sorting our workbook, lets give it a name and save it
  we'll save it in the local folder that this application is running in */
    outfile = '.\RxXL_Automation_004.xls'

/*We may want to run this several times and Excel will prompt us about replacing an existing
  file.  Since this conflicts with "automation" we'll need to turn off the Excel warnings */
    xlobj~DisplayAlerts = .false

--Save the file
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Save the workbook')
            smsg~Execute('ShowTop')
        end
    xlobj~ActiveSheet~SaveAs(outfile)

--Close the Workbook
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Close the workbook')
            smsg~Execute('ShowTop')
        end
    xlobj~WorkBooks(1)~Close(SaveAll)

--Quit Excel
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Quit Excel')
            smsg~Execute('ShowTop')
        end
    xlobj~Quit

exit

::requires 'oodwin32.cls'
::class infoDlg subclass userdialog public
::method Init
    expose width height d_title d_text
    use arg d_title, d_text
    self~Init:super()
    width = 350
    height = 45
    rc=self~CreateCenter(width,height,d_title,,,'Arial',10)

::method DefineDialog
    expose width height d_title d_text
    forward class(super) continue
    self~AddButton( 1,width-60 ,height-20,50,15,'OK','Ok','DEFAULT')
    self~AddText(1,5,350,15,d_text)

::method Ok
    self~OK:super
return self~finished


