/* RxXL_Automation_008.rex */
    parse source src
    d_title = substr(src,src~lastpos('\')+1,src~length-src~lastpos('\'))

/*
The purpose of this application is to read a ',' delimeted text file into Excel and place each
value in a cell.

This demo will be exactly like RxLoadTextXL02.rex - it's done only to demonstate that the
delimeter doesn't matter.
*/
    slowdown = .true

--Clear the screen
    call SysCls

--Before we load Excel, lets be sure the text file exists
    txtStream = .stream~new('.\RxXL1.csv')
    if txtStream~query('exists') = '' then
        do
            say 'You must Run RxXL_Automation_006.rex to create the text file.'
            exit
        end

-- Create the Excel object
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Create the Excel object')
            smsg~Execute('ShowTop')
        end
    xlobj = .OleObject~New('Excel.Application')

/*Turn the visible attribute on so that we can see what is happening
  In a production application, this can be set to false */
    xlobj~Visible = .true

--Create a new workbook with 1 worksheet
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Create a workbook with 1 worksheet')
            smsg~Execute('ShowTop')
        end
    xlobj~Application~SheetsInNewWorkBook = 1
    xlobj~WorkBooks~Add

--For this demo we will load our data into Excel starting in Row 1 Column 1(A)
--Read each line of the text file, parse the line, then load each value into a cell
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Load the data from the text file')
            smsg~Execute('ShowTop')
        end
    row = 1
    do while txtStream~lines > 0
        column = 1
        aline =  txtStream~linein
        do while aline \= ''
            parse var aline avalue','aline
            xlobj~Cells(row,column)~Value = avalue
            column = column + 1
        end
        row = row  + 1
    end
    txtStream~Close

/*Now that we have finished loading our workbook, lets give it a name and save it
  we'll save it in the default Excel folder (which will probably be My Documents'
  NOT the folder that this application is running in - you can provide any path you'd like */
    outfile = '.\myLoadedXL03.xls'

/*We may want to run this several times and Excel will prompt us about replacing an existing
  file.  Since this conflicts with "automation" we'll need to turn off the Excel warnings */
    xlobj~DisplayAlerts = .false

--Save the file
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Save the workbook')
            smsg~Execute('ShowTop')
        end
    xlobj~ActiveSheet~SaveAs(outfile)

--Close the Workbook
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Close the workbook')
            smsg~Execute('ShowTop')
        end
    xlobj~WorkBooks(1)~Close(SaveAll)

--Quit Excel
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Quit Excel')
            smsg~Execute('ShowTop')
        end
    xlobj~Quit

    say 'Congratulations, you have loaded a , delimited text file into an Excel WorkBook.'

exit

::requires 'oodwin32.cls'
::class infoDlg subclass userdialog public
::method Init
    expose width height d_title d_text
    use arg d_title, d_text
    self~Init:super()
    width = 350
    height = 45
    rc=self~CreateCenter(width,height,d_title,,,'Arial',10)

::method DefineDialog
    expose width height d_title d_text
    forward class(super) continue
    self~AddButton( 1,width-60 ,height-20,50,15,'OK','Ok','DEFAULT')
    self~AddText(1,5,350,15,d_text)

::method Ok
    self~OK:super
return self~finished


