/* RxXL_Automation_011.rex */
    parse source src
    d_title = substr(src,src~lastpos('\')+1,src~length-src~lastpos('\'))

/*
At this point you should have already run RxLoadText02.rex & RxLoadText03.rex
But you might have noticed that reading a file one line at the time, parsing each line,
and then loading each seperate value into an Excel cell, could get rather time consuming
if we were dealing with a large file.

The purpose of this application is to show that we can use the built in functionality of
Excel to 'open' the file even though it is not a .xls workbook.
*/
    slowdown = .true

--Clear the screen
    call SysCls

--Before we load Excel, lets be sure the text file exists
    txtStream = .stream~new('.\RxXL3.txt')
    if txtStream~query('exists') = '' then
        do
            say 'You must Run RxXL_Automation_006.rex to create the text file.'
            exit
        end
    thefile = txtStream~Qualify
--Remember, the data in this file is ^ delimeted

-- Create the Excel object
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Create the Excel object')
            smsg~Execute('ShowTop')
        end
    xlobj = .OleObject~New('Excel.Application')

--Define the one constant we will need
    xlDelimited = xlobj~GetConstant('xlDelimited')

/*Turn the visible attribute on so that we can see what is happening
  In a production application, this can be set to false */
    xlobj~Visible = .true

    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Directly load the delimited file')
            smsg~Execute('ShowTop')
        end
    xlobj~WorkBooks~OpenText(thefile,,1,xlDelimited,,.false,.false,.false,.false,.false,.true,'^')
--Is the delimiter a TAB?------------------------------------^      ^      ^      ^      ^     ^
--Is the delimiter a SEMICOLON?-------------------------------------^      ^      ^      ^     ^
--Is the delimiter a COMMA?------------------------------------------------^      ^      ^     ^
--Is the delimiter a SPACE?-------------------------------------------------------^      ^     ^
--Is the delimiter a OTHER?--------------------------------------------------------------^     ^
--Define the OTHER delimiter-------------------------------------------------------------------^

/*Now that we have finished loading our workbook, lets give it a name and save it
  we'll save it in the default Excel folder (which will probably be My Documents'
  NOT the folder that this application is running in - you can provide any path you'd like */
    outfile = '.\myLoadedXL06.xls'

/*We may want to run this several times and Excel will prompt us about replacing an existing
  file.  Since this conflicts with "automation" we'll need to turn off the Excel warnings */
    xlobj~DisplayAlerts = .false

--Save the file
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Save the workbook')
            smsg~Execute('ShowTop')
        end
    xlobj~ActiveSheet~SaveAs(outfile)

--Close the Workbook
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Close the workbook')
            smsg~Execute('ShowTop')
        end
    xlobj~WorkBooks(1)~Close(SaveAll)

--Quit Excel
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Quit Excel')
            smsg~Execute('ShowTop')
        end
    xlobj~Quit

exit

::requires 'oodwin32.cls'
::class infoDlg subclass userdialog public
::method Init
    expose width height d_title d_text
    use arg d_title, d_text
    self~Init:super()
    width = 350
    height = 45
    rc=self~CreateCenter(width,height,d_title,,,'Arial',10)

::method DefineDialog
    expose width height d_title d_text
    forward class(super) continue
    self~AddButton( 1,width-60 ,height-20,50,15,'OK','Ok','DEFAULT')
    self~AddText(1,5,350,15,d_text)

::method Ok
    self~OK:super
return self~finished


