/* RxXL_Automation_019.rex */
    parse source src
    d_title = substr(src,src~lastpos('\')+1,src~length-src~lastpos('\'))

/*
The purpose of this application is to demonstrate the use of ooRexx ActiveX/OLE to automate the
creation of an Excel Workbook and the necessary code to hide/unhide columns/rows.
*/
    slowdown = .true

--Clear the screen
    call SysCls

-- Create the Excel object
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Create the Excel object')
            smsg~Execute('ShowTop')
        end
    xlobj = .OleObject~New('Excel.Application')

/*Turn the visible attribute on so that we can see what is happening
  In a production application, this can be set to false */
    xlobj~Visible = .true

--Create a new workbook with 1 worksheet
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Create a workbook with 1 worksheet')
            smsg~Execute('ShowTop')
        end
    xlobj~Application~SheetsInNewWorkBook = 1
    xlobj~WorkBooks~Add

/*For the purpose of demonstration we will load random numbers into our workbook from
  columns A-T on rows 1-25.
*/
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Add some random numbers to a range of cells')
            smsg~Execute('ShowTop')
        end
    do row = 1 to 25
        do column = 1 to 20
            xlobj~Cells(row,column)~Value = random(1,5000)
        end
    end

/*
    Hide a single column (column E)
*/
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Hide column E')
            smsg~Execute('ShowTop')
        end
    xlobj~Columns('E')~Select
    xlobj~Selection~EntireColumn~Hidden = .true

    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Unhide column E')
            smsg~Execute('ShowTop')
        end

    xlobj~Selection~EntireColumn~Hidden = .false  --Remember, Column E is still selected

/*
    Hide multiple columns in one pass (G-M)
*/
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Hide columns G-M')
            smsg~Execute('ShowTop')
        end
    xlobj~Columns('G:M')~Select
    xlobj~Selection~EntireColumn~Hidden = .true

    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Unhide columns G-M')
            smsg~Execute('ShowTop')
        end

    xlobj~Selection~EntireColumn~Hidden = .false  --Remember, Columns G-M are still selected

/*
    Not surprisingly, one can hide/unhide rows is the same manner
*/
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Hide row 4')
            smsg~Execute('ShowTop')
        end
    xlobj~Rows('4')~Select
    xlobj~Selection~EntireRow~Hidden = .true

    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Unhide row 4')
            smsg~Execute('ShowTop')
        end

    xlobj~Selection~EntireRow~Hidden = .false  --Remember, Row 4 is still selected

/*
    Hide multiple rows in one pass (3-8)
*/

    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Hide rows 3-8')
            smsg~Execute('ShowTop')
        end
    xlobj~Rows('3:8')~Select
    xlobj~Selection~EntireRow~Hidden = .true

/*
    There may be a case where the hidden rows/columns are not known and the desire is to unhide
    all rows/columns.  In the example below we'll select all rows between our known first row and our
    known last row (see the first example in this series for the routine to use to determine the maximum
    row and column used in a Workbook when the value is unknown.
    We'll just select all rows between 1 & 25 and unhide them (same could be done with columns).
*/

    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Unhide all (any) hidden rows')
            smsg~Execute('ShowTop')
        end

    xlobj~Rows('1:25')~Select
    xlobj~Selection~EntireRow~Hidden = .false

/*
    Let's go back an hide columns B,E,I,O,S and then save our Workbook
*/
    c_hides = 'B E I O S'
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Hide columns B,E,I,O,S')
            smsg~Execute('ShowTop')
        end
    do i = 1 to c_hides~words
        col = c_hides~word(i)
        xlobj~Columns(col)~Select
        xlobj~Selection~EntireColumn~Hidden = .true
    end

/*
    To be a good steward of Excel, the programmer should always leave the cursor in Cell A1
*/
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Leave cursor in A1')
            smsg~Execute('ShowTop')
        end

    xlobj~Range("A1")~Select

/*Now that we have finished creating our workbook, lets give it a name and save it
  we'll save it in the default Excel folder (which will probably be My Documents'
  NOT the folder that this application is running in - you can provide any path you'd like */
    outfile = '.\RxXL_Automation_019.xls'

/*We may want to run this several times and Excel will prompt us about replacing an existing
  file.  Since this conflicts with "automation" we'll need to turn off the Excel warnings */
    xlobj~DisplayAlerts = .false

--Save the file
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Save the workbook')
            smsg~Execute('ShowTop')
        end
    xlobj~ActiveSheet~SaveAs(outfile)

--Close the Workbook
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Close the workbook')
            smsg~Execute('ShowTop')
        end
    xlobj~WorkBooks(1)~Close(SaveAll)

--Quit Excel
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Quit Excel')
            smsg~Execute('ShowTop')
        end
    xlobj~Quit

    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Congratulations')
            smsg~Execute('ShowTop')
        end
exit

::requires 'oodwin32.cls'
::class infoDlg subclass userdialog public
::method Init
    expose width height d_title d_text
    use arg d_title, d_text
    self~Init:super()
    width = 350
    height = 45
    rc=self~CreateCenter(width,height,d_title,,,'Arial',10)

::method DefineDialog
    expose width height d_title d_text
    forward class(super) continue
    self~AddButton( 1,width-60 ,height-20,50,15,'OK','Ok','DEFAULT')
    self~AddText(1,5,350,15,d_text)

::method Ok
    self~OK:super
return self~finished


