/* RxXL_Automation_020.rex */
    parse source src
    d_title = substr(src,src~lastpos('\')+1,src~length-src~lastpos('\'))

/*
The purpose of this application is to demonstrate the use of ooRexx ActiveX/OLE to automate the
creation of an Excel Workbook and the necessary code to copy Worksheets before and after an existing
Worksheet.
*/
    slowdown = .true

--Clear the screen
    call SysCls

-- Create the Excel object
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Create the Excel object')
            smsg~Execute('ShowTop')
        end
    xlobj = .OleObject~New('Excel.Application')

/*Turn the visible attribute on so that we can see what is happening
  In a production application, this can be set to false */
    xlobj~Visible = .true

--Create a new workbook with 1 worksheet
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Create a workbook with 1 worksheet')
            smsg~Execute('ShowTop')
        end
    xlobj~Application~SheetsInNewWorkBook = 1
    xlobj~WorkBooks~Add

--Rename the one sheet to "MasterSheet"
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Rename the one existing worksheet')
            smsg~Execute('ShowTop')
        end
    xlobj~Sheets('Sheet1')~Name = 'MasterSheet'

-- For the purpose of demonstration we will add a value to cell A1 of
-- the only (current) worksheet in this workbook
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Add a value to cell A1')
            smsg~Execute('ShowTop')
        end
    xlobj~Cells(1,1)~Value = 'This is our Template worksheet'

-- Autofit the column so that we can see that the format is copied as well
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Format the column width')
            smsg~Execute('ShowTop')
        end
    xlobj~Range('A:A')~Columns~AutoFit

-- Copy the MasterSheet to a new worksheet "before" MasterSheet
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Make a copy of MasterSheet "BEFORE" the existing MasterSheet')
            smsg~Execute('ShowTop')
        end
    msgs = .array~of(xlobj~Sheets('MasterSheet'))
    xlObj~Sheets("MasterSheet")~Unknown('Copy',msgs)

-- Rename the new worksheet
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Rename the added worksheet')
            smsg~Execute('ShowTop')
        end
    xlObj~Sheets("MasterSheet (2)")~Name = "Before_Sheet"

-- Copy the MasterSheet to a new worksheet "after" MasterSheet
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Make a copy of MasterSheet "AFTER" the existing MasterSheet')
            smsg~Execute('ShowTop')
        end

-- Following code supplied by Scott.G.Kinsella, note the .nil as the first parameter in the
-- array element
    msgs = .array~of(.nil,xlobj~Sheets('MasterSheet'))
    xlObj~Sheets("MasterSheet")~Unknown('Copy',msgs)

-- Rename the new worksheet
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Rename the added worksheet')
            smsg~Execute('ShowTop')
        end
    xlObj~Sheets("MasterSheet (2)")~Name = "After_Sheet"

-- Go back to our "Before" sheet and change the value in A1
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Select the Before sheet and change the value in A1')
            smsg~Execute('ShowTop')
        end
    xlobj~Sheets("Before_Sheet")~Select
    xlobj~Range("A1")~Select
    xlobj~ActiveCell~FormulaR1C1 = "This is our Before sheet"

-- Go back to our "After" sheet and change the value in A1
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Select the After sheet and change the value in A1')
            smsg~Execute('ShowTop')
        end
    xlobj~Sheets("After_Sheet")~Select
    xlobj~Range("A1")~Select
    xlobj~ActiveCell~FormulaR1C1 = "This is our After sheet"

-- To be a good steward of Excel, the programmer should always leave the cursor in Cell A1
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Leave cursor in A1 of the MasterSheet')
            smsg~Execute('ShowTop')
        end
    xlobj~Sheets("MasterSheet")~Select
    xlobj~Range("A1")~Select

/*Now that we have finished creating our workbook, lets give it a name and save it
  we'll save it in the default Excel folder (which will probably be My Documents'
  NOT the folder that this application is running in - you can provide any path you'd like */
    outfile = '.\RxXL_Automation_020.xls'

/*We may want to run this several times and Excel will prompt us about replacing an existing
  file.  Since this conflicts with "automation" we'll need to turn off the Excel warnings */
    xlobj~DisplayAlerts = .false

--Save the file
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Save the workbook')
            smsg~Execute('ShowTop')
        end
    xlobj~ActiveSheet~SaveAs(outfile)

--Close the Workbook
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Close the workbook')
            smsg~Execute('ShowTop')
        end
    xlobj~WorkBooks(1)~Close(SaveAll)

--Quit Excel
    if slowdown then
        do
            smsg = .infoDlg~new(d_title,'Quit Excel')
            smsg~Execute('ShowTop')
        end
    xlobj~Quit

exit

::requires 'oodwin32.cls'
::class infoDlg subclass userdialog public
::method Init
    expose width height d_title d_text
    use arg d_title, d_text
    self~Init:super()
    width = 350
    height = 45
    rc=self~CreateCenter(width,height,d_title,,,'Arial',10)

::method DefineDialog
    expose width height d_title d_text
    forward class(super) continue
    self~AddButton( 1,width-60 ,height-20,50,15,'OK','Ok','DEFAULT')
    self~AddText(1,5,350,15,d_text)

::method Ok
    self~OK:super
return self~finished


