/* RxXL_Automation_020.rex */

/*
The purpose of this application is to demonstrate the use of ooRexx ActiveX/OLE to automate the
creation of an Excel Workbook and the necessary code to copy Worksheets before and after an existing
Worksheet.
*/

--Clear the screen
    call SysCls

-- Create the Excel object
    xlobj = .OleObject~New('Excel.Application')

/*Turn the visible attribute on so that we can see what is happening
  In a production application, this can be set to false */
    xlobj~Visible = .true

--Create a new workbook with 1 worksheet
    xlobj~Application~SheetsInNewWorkBook = 1
    xlobj~WorkBooks~Add

--Rename the one sheet to "MasterSheet"
    xlobj~Sheets('Sheet1')~Name = 'MasterSheet'

-- For the purpose of demonstration we will add a value to cell A1 of
-- the only (current) worksheet in this workbook
    xlobj~Cells(1,1)~Value = 'This is our Template worksheet'

-- Autofit the column so that we can see that the format is copied as well
    xlobj~Range('A:A')~Columns~AutoFit

-- Copy the MasterSheet to a new worksheet "before" MasterSheet
    msgs = .array~of(xlobj~Sheets('MasterSheet'))
    xlObj~Sheets("MasterSheet")~Unknown('Copy',msgs)

-- Rename the new worksheet
    xlObj~Sheets("MasterSheet (2)")~Name = "Before_Sheet"

-- Copy the MasterSheet to a new worksheet "after" MasterSheet
-- Following code supplied by Scott.G.Kinsella, note the .nil as the first parameter in the
-- array element
    msgs = .array~of(.nil,xlobj~Sheets('MasterSheet'))
    xlObj~Sheets("MasterSheet")~Unknown('Copy',msgs)

-- Rename the new worksheet
    xlObj~Sheets("MasterSheet (2)")~Name = "After_Sheet"

-- Go back to our "Before" sheet and change the value in A1
    xlobj~Sheets("Before_Sheet")~Select
    xlobj~Range("A1")~Select
    xlobj~ActiveCell~FormulaR1C1 = "This is our Before sheet"

-- Go back to our "After" sheet and change the value in A1
    xlobj~Sheets("After_Sheet")~Select
    xlobj~Range("A1")~Select
    xlobj~ActiveCell~FormulaR1C1 = "This is our After sheet"

-- To be a good steward of Excel, the programmer should always leave the cursor in Cell A1
    xlobj~Sheets("MasterSheet")~Select
    xlobj~Range("A1")~Select

/*Now that we have finished creating our workbook, lets give it a name and save it
  we'll save it in the default Excel folder (which will probably be My Documents'
  NOT the folder that this application is running in - you can provide any path you'd like */
    outfile = '.\RxXL_Automation_020a.xls'

/*We may want to run this several times and Excel will prompt us about replacing an existing
  file.  Since this conflicts with "automation" we'll need to turn off the Excel warnings */
    xlobj~DisplayAlerts = .false

--Save the file
    xlobj~ActiveSheet~SaveAs(outfile)

--Close the Workbook
    xlobj~WorkBooks(1)~Close(SaveAll)

--Quit Excel
    xlobj~Quit
exit


